/* The table of wrapper functions to populate */

/* This file is generated and should not be modified.  See the makewrappers
 * script if you want to modify this. */
typedef struct {
	char *name;		/* the name */
	void (**real)(void);	/* the underlying syscall */
	void (*wrapper)(void);	/* the wrapper from guts/name.c */
	char *version;		/* the version, if we know and care */
} pseudo_function;

static pseudo_function pseudo_functions[] = {
	{ /* int __fxstat(int ver, int fd, struct stat *buf); */
		"__fxstat",
		(void (**)(void)) &real___fxstat,
		(void (*)(void)) wrap___fxstat,
		NULL
	},
	{ /* int __fxstat64(int ver, int fd, struct stat64 *buf); */
		"__fxstat64",
		(void (**)(void)) &real___fxstat64,
		(void (*)(void)) wrap___fxstat64,
		NULL
	},
	{ /* int __fxstatat(int ver, int dirfd, const char *path, struct stat *buf, int flags); */
		"__fxstatat",
		(void (**)(void)) &real___fxstatat,
		(void (*)(void)) wrap___fxstatat,
		NULL
	},
	{ /* int __fxstatat64(int ver, int dirfd, const char *path, struct stat64 *buf, int flags); */
		"__fxstatat64",
		(void (**)(void)) &real___fxstatat64,
		(void (*)(void)) wrap___fxstatat64,
		NULL
	},
	{ /* int __lxstat(int ver, const char *path, struct stat *buf); */
		"__lxstat",
		(void (**)(void)) &real___lxstat,
		(void (*)(void)) wrap___lxstat,
		NULL
	},
	{ /* int __lxstat64(int ver, const char *path, struct stat64 *buf); */
		"__lxstat64",
		(void (**)(void)) &real___lxstat64,
		(void (*)(void)) wrap___lxstat64,
		NULL
	},
	{ /* int __openat64_2(int dirfd, const char *path, int flags); */
		"__openat64_2",
		(void (**)(void)) &real___openat64_2,
		(void (*)(void)) wrap___openat64_2,
		NULL
	},
	{ /* int __openat_2(int dirfd, const char *path, int flags); */
		"__openat_2",
		(void (**)(void)) &real___openat_2,
		(void (*)(void)) wrap___openat_2,
		NULL
	},
	{ /* int __xmknod(int ver, const char *path, mode_t mode, dev_t *dev); */
		"__xmknod",
		(void (**)(void)) &real___xmknod,
		(void (*)(void)) wrap___xmknod,
		NULL
	},
	{ /* int __xmknodat(int ver, int dirfd, const char *path, mode_t mode, dev_t *dev); */
		"__xmknodat",
		(void (**)(void)) &real___xmknodat,
		(void (*)(void)) wrap___xmknodat,
		NULL
	},
	{ /* int __xstat(int ver, const char *path, struct stat *buf); */
		"__xstat",
		(void (**)(void)) &real___xstat,
		(void (*)(void)) wrap___xstat,
		NULL
	},
	{ /* int __xstat64(int ver, const char *path, struct stat64 *buf); */
		"__xstat64",
		(void (**)(void)) &real___xstat64,
		(void (*)(void)) wrap___xstat64,
		NULL
	},
	{ /* int access(const char *path, int mode); */
		"access",
		(void (**)(void)) &real_access,
		(void (*)(void)) wrap_access,
		NULL
	},
	{ /* int acct(const char *path); */
		"acct",
		(void (**)(void)) &real_acct,
		(void (*)(void)) wrap_acct,
		NULL
	},
	{ /* int bind(int sockfd, const struct sockaddr *addr, socklen_t addrlen); */
		"bind",
		(void (**)(void)) &real_bind,
		(void (*)(void)) wrap_bind,
		NULL
	},
	{ /* char *canonicalize_file_name(const char *filename); */
		"canonicalize_file_name",
		(void (**)(void)) &real_canonicalize_file_name,
		(void (*)(void)) wrap_canonicalize_file_name,
		NULL
	},
	{ /* int capset(cap_user_header_t hdrp, const cap_user_data_t datap); */
		"capset",
		(void (**)(void)) &real_capset,
		(void (*)(void)) wrap_capset,
		NULL
	},
	{ /* int chdir(const char *path); */
		"chdir",
		(void (**)(void)) &real_chdir,
		(void (*)(void)) wrap_chdir,
		NULL
	},
	{ /* int chmod(const char *path, mode_t mode); */
		"chmod",
		(void (**)(void)) &real_chmod,
		(void (*)(void)) wrap_chmod,
		NULL
	},
	{ /* int chown(const char *path, uid_t owner, gid_t group); */
		"chown",
		(void (**)(void)) &real_chown,
		(void (*)(void)) wrap_chown,
		NULL
	},
	{ /* int chroot(const char *path); */
		"chroot",
		(void (**)(void)) &real_chroot,
		(void (*)(void)) wrap_chroot,
		NULL
	},
	{ /* int clone(int (*fn)(void *), void *child_stack, int flags, void *arg, va_list ap); */
		"clone",
		(void (**)(void)) &real_clone,
		(void (*)(void)) wrap_clone,
		NULL
	},
	{ /* int close(int fd); */
		"close",
		(void (**)(void)) &real_close,
		(void (*)(void)) wrap_close,
		NULL
	},
	{ /* int close_range(unsigned int lowfd, unsigned int maxfd, int flags); */
		"close_range",
		(void (**)(void)) &real_close_range,
		(void (*)(void)) wrap_close_range,
		NULL
	},
	{ /* int closedir(DIR *dirp); */
		"closedir",
		(void (**)(void)) &real_closedir,
		(void (*)(void)) wrap_closedir,
		NULL
	},
	{ /* void closefrom(int fd); */
		"closefrom",
		(void (**)(void)) &real_closefrom,
		(void (*)(void)) wrap_closefrom,
		NULL
	},
	{ /* int creat(const char *path, mode_t mode); */
		"creat",
		(void (**)(void)) &real_creat,
		(void (*)(void)) wrap_creat,
		NULL
	},
	{ /* int creat64(const char *path, mode_t mode); */
		"creat64",
		(void (**)(void)) &real_creat64,
		(void (*)(void)) wrap_creat64,
		NULL
	},
	{ /* int dup(int fd); */
		"dup",
		(void (**)(void)) &real_dup,
		(void (*)(void)) wrap_dup,
		NULL
	},
	{ /* int dup2(int oldfd, int newfd); */
		"dup2",
		(void (**)(void)) &real_dup2,
		(void (*)(void)) wrap_dup2,
		NULL
	},
	{ /* int eaccess(const char *path, int mode); */
		"eaccess",
		(void (**)(void)) &real_eaccess,
		(void (*)(void)) wrap_eaccess,
		NULL
	},
	{ /* void endgrent(void); */
		"endgrent",
		(void (**)(void)) &real_endgrent,
		(void (*)(void)) wrap_endgrent,
		NULL
	},
	{ /* void endpwent(void); */
		"endpwent",
		(void (**)(void)) &real_endpwent,
		(void (*)(void)) wrap_endpwent,
		NULL
	},
	{ /* int euidaccess(const char *path, int mode); */
		"euidaccess",
		(void (**)(void)) &real_euidaccess,
		(void (*)(void)) wrap_euidaccess,
		NULL
	},
	{ /* int execl(const char *file, const char *arg, va_list ap); */
		"execl",
		(void (**)(void)) &real_execl,
		(void (*)(void)) wrap_execl,
		NULL
	},
	{ /* int execle(const char *file, const char *arg, va_list ap); */
		"execle",
		(void (**)(void)) &real_execle,
		(void (*)(void)) wrap_execle,
		NULL
	},
	{ /* int execlp(const char *file, const char *arg, va_list ap); */
		"execlp",
		(void (**)(void)) &real_execlp,
		(void (*)(void)) wrap_execlp,
		NULL
	},
	{ /* int execv(const char *file, char *const *argv); */
		"execv",
		(void (**)(void)) &real_execv,
		(void (*)(void)) wrap_execv,
		NULL
	},
	{ /* int execve(const char *file, char *const *argv, char *const *envp); */
		"execve",
		(void (**)(void)) &real_execve,
		(void (*)(void)) wrap_execve,
		NULL
	},
	{ /* int execvp(const char *file, char *const *argv); */
		"execvp",
		(void (**)(void)) &real_execvp,
		(void (*)(void)) wrap_execvp,
		NULL
	},
	{ /* int faccessat(int dirfd, const char *path, int mode, int flags); */
		"faccessat",
		(void (**)(void)) &real_faccessat,
		(void (*)(void)) wrap_faccessat,
		NULL
	},
	{ /* int faccessat2(int dirfd, const char *path, int mode, int flags); */
		"faccessat2",
		(void (**)(void)) &real_faccessat2,
		(void (*)(void)) wrap_faccessat2,
		NULL
	},
	{ /* int fchdir(int dirfd); */
		"fchdir",
		(void (**)(void)) &real_fchdir,
		(void (*)(void)) wrap_fchdir,
		NULL
	},
	{ /* int fchmod(int fd, mode_t mode); */
		"fchmod",
		(void (**)(void)) &real_fchmod,
		(void (*)(void)) wrap_fchmod,
		NULL
	},
	{ /* int fchmodat(int dirfd, const char *path, mode_t mode, int flags); */
		"fchmodat",
		(void (**)(void)) &real_fchmodat,
		(void (*)(void)) wrap_fchmodat,
		NULL
	},
	{ /* int fchown(int fd, uid_t owner, gid_t group); */
		"fchown",
		(void (**)(void)) &real_fchown,
		(void (*)(void)) wrap_fchown,
		NULL
	},
	{ /* int fchownat(int dirfd, const char *path, uid_t owner, gid_t group, int flags); */
		"fchownat",
		(void (**)(void)) &real_fchownat,
		(void (*)(void)) wrap_fchownat,
		NULL
	},
	{ /* int fclose(FILE *fp); */
		"fclose",
		(void (**)(void)) &real_fclose,
		(void (*)(void)) wrap_fclose,
		NULL
	},
	{ /* int fcntl(int fd, int cmd, ... { struct flock *lock }); */
		"fcntl",
		(void (**)(void)) &real_fcntl,
		(void (*)(void)) wrap_fcntl,
		NULL
	},
	{ /* int fcntl64(int fd, int cmd, ... { struct flock *lock }); */
		"fcntl64",
		(void (**)(void)) &real_fcntl64,
		(void (*)(void)) wrap_fcntl64,
		NULL
	},
	{ /* int fdatasync(int fd); */
		"fdatasync",
		(void (**)(void)) &real_fdatasync,
		(void (*)(void)) wrap_fdatasync,
		NULL
	},
	{ /* ssize_t fgetxattr(int filedes, const char *name, void *value, size_t size); */
		"fgetxattr",
		(void (**)(void)) &real_fgetxattr,
		(void (*)(void)) wrap_fgetxattr,
		"GLIBC_2.3"
	},
	{ /* ssize_t flistxattr(int filedes, char *list, size_t size); */
		"flistxattr",
		(void (**)(void)) &real_flistxattr,
		(void (*)(void)) wrap_flistxattr,
		"GLIBC_2.3"
	},
	{ /* FILE *fopen(const char *path, const char *mode); */
		"fopen",
		(void (**)(void)) &real_fopen,
		(void (*)(void)) wrap_fopen,
		NULL
	},
	{ /* FILE *fopen64(const char *path, const char *mode); */
		"fopen64",
		(void (**)(void)) &real_fopen64,
		(void (*)(void)) wrap_fopen64,
		NULL
	},
	{ /* int fork(void); */
		"fork",
		(void (**)(void)) &real_fork,
		(void (*)(void)) wrap_fork,
		NULL
	},
	{ /* int fremovexattr(int filedes, const char *name); */
		"fremovexattr",
		(void (**)(void)) &real_fremovexattr,
		(void (*)(void)) wrap_fremovexattr,
		"GLIBC_2.3"
	},
	{ /* FILE *freopen(const char *path, const char *mode, FILE *stream); */
		"freopen",
		(void (**)(void)) &real_freopen,
		(void (*)(void)) wrap_freopen,
		NULL
	},
	{ /* FILE *freopen64(const char *path, const char *mode, FILE *stream); */
		"freopen64",
		(void (**)(void)) &real_freopen64,
		(void (*)(void)) wrap_freopen64,
		NULL
	},
	{ /* int fsetxattr(int filedes, const char *name, const void *value, size_t size, int xflags); */
		"fsetxattr",
		(void (**)(void)) &real_fsetxattr,
		(void (*)(void)) wrap_fsetxattr,
		"GLIBC_2.3"
	},
	{ /* int fstat(int fd, struct stat *buf); */
		"fstat",
		(void (**)(void)) &real_fstat,
		(void (*)(void)) wrap_fstat,
		NULL
	},
	{ /* int fstat64(int fd, struct stat64 *buf); */
		"fstat64",
		(void (**)(void)) &real_fstat64,
		(void (*)(void)) wrap_fstat64,
		NULL
	},
	{ /* int fstatat(int dirfd, const char *path, struct stat *buf, int flags); */
		"fstatat",
		(void (**)(void)) &real_fstatat,
		(void (*)(void)) wrap_fstatat,
		NULL
	},
	{ /* int fstatat64(int dirfd, const char *path, struct stat64 *buf, int flags); */
		"fstatat64",
		(void (**)(void)) &real_fstatat64,
		(void (*)(void)) wrap_fstatat64,
		NULL
	},
	{ /* int fsync(int fd); */
		"fsync",
		(void (**)(void)) &real_fsync,
		(void (*)(void)) wrap_fsync,
		NULL
	},
	{ /* FTS *fts_open(char * const *path_argv, int options, int (*compar)(const FTSENT **, const FTSENT **)); */
		"fts_open",
		(void (**)(void)) &real_fts_open,
		(void (*)(void)) wrap_fts_open,
		NULL
	},
	{ /* int ftw(const char *path, int (*fn)(const char *, const struct stat *, int), int nopenfd); */
		"ftw",
		(void (**)(void)) &real_ftw,
		(void (*)(void)) wrap_ftw,
		NULL
	},
	{ /* int ftw64(const char *path, int (*fn)(const char *, const struct stat64 *, int), int nopenfd); */
		"ftw64",
		(void (**)(void)) &real_ftw64,
		(void (*)(void)) wrap_ftw64,
		NULL
	},
	{ /* char *get_current_dir_name(void); */
		"get_current_dir_name",
		(void (**)(void)) &real_get_current_dir_name,
		(void (*)(void)) wrap_get_current_dir_name,
		NULL
	},
	{ /* char *getcwd(char *buf, size_t size); */
		"getcwd",
		(void (**)(void)) &real_getcwd,
		(void (*)(void)) wrap_getcwd,
		NULL
	},
	{ /* gid_t getegid(void); */
		"getegid",
		(void (**)(void)) &real_getegid,
		(void (*)(void)) wrap_getegid,
		NULL
	},
	{ /* uid_t geteuid(void); */
		"geteuid",
		(void (**)(void)) &real_geteuid,
		(void (*)(void)) wrap_geteuid,
		NULL
	},
	{ /* gid_t getgid(void); */
		"getgid",
		(void (**)(void)) &real_getgid,
		(void (*)(void)) wrap_getgid,
		NULL
	},
	{ /* struct group *getgrent(void); */
		"getgrent",
		(void (**)(void)) &real_getgrent,
		(void (*)(void)) wrap_getgrent,
		NULL
	},
	{ /* int getgrent_r(struct group *gbuf, char *buf, size_t buflen, struct group **gbufp); */
		"getgrent_r",
		(void (**)(void)) &real_getgrent_r,
		(void (*)(void)) wrap_getgrent_r,
		NULL
	},
	{ /* struct group *getgrgid(gid_t gid); */
		"getgrgid",
		(void (**)(void)) &real_getgrgid,
		(void (*)(void)) wrap_getgrgid,
		NULL
	},
	{ /* int getgrgid_r(gid_t gid, struct group *gbuf, char *buf, size_t buflen, struct group **gbufp); */
		"getgrgid_r",
		(void (**)(void)) &real_getgrgid_r,
		(void (*)(void)) wrap_getgrgid_r,
		NULL
	},
	{ /* struct group *getgrnam(const char *name); */
		"getgrnam",
		(void (**)(void)) &real_getgrnam,
		(void (*)(void)) wrap_getgrnam,
		NULL
	},
	{ /* int getgrnam_r(const char *name, struct group *gbuf, char *buf, size_t buflen, struct group **gbufp); */
		"getgrnam_r",
		(void (**)(void)) &real_getgrnam_r,
		(void (*)(void)) wrap_getgrnam_r,
		NULL
	},
	{ /* int getgrouplist(const char *user, gid_t group, gid_t *groups, int *ngroups); */
		"getgrouplist",
		(void (**)(void)) &real_getgrouplist,
		(void (*)(void)) wrap_getgrouplist,
		NULL
	},
	{ /* int getgroups(int size, gid_t *list); */
		"getgroups",
		(void (**)(void)) &real_getgroups,
		(void (*)(void)) wrap_getgroups,
		NULL
	},
	{ /* int getpw(uid_t uid, char *buf); */
		"getpw",
		(void (**)(void)) &real_getpw,
		(void (*)(void)) wrap_getpw,
		NULL
	},
	{ /* struct passwd *getpwent(void); */
		"getpwent",
		(void (**)(void)) &real_getpwent,
		(void (*)(void)) wrap_getpwent,
		NULL
	},
	{ /* int getpwent_r(struct passwd *pwbuf, char *buf, size_t buflen, struct passwd **pwbufp); */
		"getpwent_r",
		(void (**)(void)) &real_getpwent_r,
		(void (*)(void)) wrap_getpwent_r,
		NULL
	},
	{ /* struct passwd *getpwnam(const char *name); */
		"getpwnam",
		(void (**)(void)) &real_getpwnam,
		(void (*)(void)) wrap_getpwnam,
		NULL
	},
	{ /* int getpwnam_r(const char *name, struct passwd *pwbuf, char *buf, size_t buflen, struct passwd **pwbufp); */
		"getpwnam_r",
		(void (**)(void)) &real_getpwnam_r,
		(void (*)(void)) wrap_getpwnam_r,
		NULL
	},
	{ /* struct passwd *getpwuid(uid_t uid); */
		"getpwuid",
		(void (**)(void)) &real_getpwuid,
		(void (*)(void)) wrap_getpwuid,
		NULL
	},
	{ /* int getpwuid_r(uid_t uid, struct passwd *pwbuf, char *buf, size_t buflen, struct passwd **pwbufp); */
		"getpwuid_r",
		(void (**)(void)) &real_getpwuid_r,
		(void (*)(void)) wrap_getpwuid_r,
		NULL
	},
	{ /* int getresgid(gid_t *rgid, gid_t *egid, gid_t *sgid); */
		"getresgid",
		(void (**)(void)) &real_getresgid,
		(void (*)(void)) wrap_getresgid,
		NULL
	},
	{ /* int getresuid(uid_t *ruid, uid_t *euid, uid_t *suid); */
		"getresuid",
		(void (**)(void)) &real_getresuid,
		(void (*)(void)) wrap_getresuid,
		NULL
	},
	{ /* uid_t getuid(void); */
		"getuid",
		(void (**)(void)) &real_getuid,
		(void (*)(void)) wrap_getuid,
		NULL
	},
	{ /* char *getwd(char *buf); */
		"getwd",
		(void (**)(void)) &real_getwd,
		(void (*)(void)) wrap_getwd,
		NULL
	},
	{ /* ssize_t getxattr(const char *path, const char *name, void *value, size_t size); */
		"getxattr",
		(void (**)(void)) &real_getxattr,
		(void (*)(void)) wrap_getxattr,
		"GLIBC_2.3"
	},
	{ /* int glob(const char *pattern, int flags, int (*errfunc)(const char *, int), glob_t *pglob); */
		"glob",
		(void (**)(void)) &real_glob,
		(void (*)(void)) wrap_glob,
		NULL
	},
	{ /* int glob64(const char *pattern, int flags, int (*errfunc)(const char *, int), glob64_t *pglob); */
		"glob64",
		(void (**)(void)) &real_glob64,
		(void (*)(void)) wrap_glob64,
		NULL
	},
	{ /* int lchmod(const char *path, mode_t mode); */
		"lchmod",
		(void (**)(void)) &real_lchmod,
		(void (*)(void)) wrap_lchmod,
		NULL
	},
	{ /* int lchown(const char *path, uid_t owner, gid_t group); */
		"lchown",
		(void (**)(void)) &real_lchown,
		(void (*)(void)) wrap_lchown,
		NULL
	},
	{ /* int lckpwdf(void); */
		"lckpwdf",
		(void (**)(void)) &real_lckpwdf,
		(void (*)(void)) wrap_lckpwdf,
		NULL
	},
	{ /* ssize_t lgetxattr(const char *path, const char *name, void *value, size_t size); */
		"lgetxattr",
		(void (**)(void)) &real_lgetxattr,
		(void (*)(void)) wrap_lgetxattr,
		"GLIBC_2.3"
	},
	{ /* int link(const char *oldname, const char *newname); */
		"link",
		(void (**)(void)) &real_link,
		(void (*)(void)) wrap_link,
		NULL
	},
	{ /* int linkat(int olddirfd, const char *oldname, int newdirfd, const char *newname, int flags); */
		"linkat",
		(void (**)(void)) &real_linkat,
		(void (*)(void)) wrap_linkat,
		NULL
	},
	{ /* ssize_t listxattr(const char *path, char *list, size_t size); */
		"listxattr",
		(void (**)(void)) &real_listxattr,
		(void (*)(void)) wrap_listxattr,
		"GLIBC_2.3"
	},
	{ /* ssize_t llistxattr(const char *path, char *list, size_t size); */
		"llistxattr",
		(void (**)(void)) &real_llistxattr,
		(void (*)(void)) wrap_llistxattr,
		"GLIBC_2.3"
	},
	{ /* int lremovexattr(const char *path, const char *name); */
		"lremovexattr",
		(void (**)(void)) &real_lremovexattr,
		(void (*)(void)) wrap_lremovexattr,
		"GLIBC_2.3"
	},
	{ /* int lsetxattr(const char *path, const char *name, const void *value, size_t size, int xflags); */
		"lsetxattr",
		(void (**)(void)) &real_lsetxattr,
		(void (*)(void)) wrap_lsetxattr,
		"GLIBC_2.3"
	},
	{ /* int lstat(const char *path, struct stat *buf); */
		"lstat",
		(void (**)(void)) &real_lstat,
		(void (*)(void)) wrap_lstat,
		NULL
	},
	{ /* int lstat64(const char *path, struct stat64 *buf); */
		"lstat64",
		(void (**)(void)) &real_lstat64,
		(void (*)(void)) wrap_lstat64,
		NULL
	},
	{ /* int lutimes(const char *path, const struct timeval *tv); */
		"lutimes",
		(void (**)(void)) &real_lutimes,
		(void (*)(void)) wrap_lutimes,
		NULL
	},
	{ /* int mkdir(const char *path, mode_t mode); */
		"mkdir",
		(void (**)(void)) &real_mkdir,
		(void (*)(void)) wrap_mkdir,
		NULL
	},
	{ /* int mkdirat(int dirfd, const char *path, mode_t mode); */
		"mkdirat",
		(void (**)(void)) &real_mkdirat,
		(void (*)(void)) wrap_mkdirat,
		NULL
	},
	{ /* char *mkdtemp(char *template); */
		"mkdtemp",
		(void (**)(void)) &real_mkdtemp,
		(void (*)(void)) wrap_mkdtemp,
		NULL
	},
	{ /* int mkfifo(const char *path, mode_t mode); */
		"mkfifo",
		(void (**)(void)) &real_mkfifo,
		(void (*)(void)) wrap_mkfifo,
		NULL
	},
	{ /* int mkfifoat(int dirfd, const char *path, mode_t mode); */
		"mkfifoat",
		(void (**)(void)) &real_mkfifoat,
		(void (*)(void)) wrap_mkfifoat,
		NULL
	},
	{ /* int mknod(const char *path, mode_t mode, dev_t dev); */
		"mknod",
		(void (**)(void)) &real_mknod,
		(void (*)(void)) wrap_mknod,
		NULL
	},
	{ /* int mknodat(int dirfd, const char *path, mode_t mode, dev_t dev); */
		"mknodat",
		(void (**)(void)) &real_mknodat,
		(void (*)(void)) wrap_mknodat,
		NULL
	},
	{ /* int mkostemp(char *template, int oflags); */
		"mkostemp",
		(void (**)(void)) &real_mkostemp,
		(void (*)(void)) wrap_mkostemp,
		NULL
	},
	{ /* int mkostemp64(char *template, int oflags); */
		"mkostemp64",
		(void (**)(void)) &real_mkostemp64,
		(void (*)(void)) wrap_mkostemp64,
		NULL
	},
	{ /* int mkostemps(char *template, int suffixlen, int oflags); */
		"mkostemps",
		(void (**)(void)) &real_mkostemps,
		(void (*)(void)) wrap_mkostemps,
		NULL
	},
	{ /* int mkstemp(char *template); */
		"mkstemp",
		(void (**)(void)) &real_mkstemp,
		(void (*)(void)) wrap_mkstemp,
		NULL
	},
	{ /* int mkstemp64(char *template); */
		"mkstemp64",
		(void (**)(void)) &real_mkstemp64,
		(void (*)(void)) wrap_mkstemp64,
		NULL
	},
	{ /* int mkstemps(char *template, int suffixlen); */
		"mkstemps",
		(void (**)(void)) &real_mkstemps,
		(void (*)(void)) wrap_mkstemps,
		NULL
	},
	{ /* char *mktemp(char *template); */
		"mktemp",
		(void (**)(void)) &real_mktemp,
		(void (*)(void)) wrap_mktemp,
		NULL
	},
	{ /* int msync(void *addr, size_t length, int flags); */
		"msync",
		(void (**)(void)) &real_msync,
		(void (*)(void)) wrap_msync,
		NULL
	},
	{ /* int nftw(const char *path, int (*fn)(const char *, const struct stat *, int, struct FTW *), int nopenfd, int flag); */
		"nftw",
		(void (**)(void)) &real_nftw,
		(void (*)(void)) wrap_nftw,
		NULL
	},
	{ /* int nftw64(const char *path, int (*fn)(const char *, const struct stat64 *, int, struct FTW *), int nopenfd, int flag); */
		"nftw64",
		(void (**)(void)) &real_nftw64,
		(void (*)(void)) wrap_nftw64,
		NULL
	},
	{ /* int open(const char *path, int flags, ... { mode_t mode }); */
		"open",
		(void (**)(void)) &real_open,
		(void (*)(void)) wrap_open,
		NULL
	},
	{ /* int open64(const char *path, int flags, ... { mode_t mode }); */
		"open64",
		(void (**)(void)) &real_open64,
		(void (*)(void)) wrap_open64,
		NULL
	},
	{ /* int openat(int dirfd, const char *path, int flags, ... { mode_t mode }); */
		"openat",
		(void (**)(void)) &real_openat,
		(void (*)(void)) wrap_openat,
		NULL
	},
	{ /* int openat64(int dirfd, const char *path, int flags, ... { mode_t mode }); */
		"openat64",
		(void (**)(void)) &real_openat64,
		(void (*)(void)) wrap_openat64,
		NULL
	},
	{ /* DIR *opendir(const char *path); */
		"opendir",
		(void (**)(void)) &real_opendir,
		(void (*)(void)) wrap_opendir,
		NULL
	},
	{ /* long pathconf(const char *path, int name); */
		"pathconf",
		(void (**)(void)) &real_pathconf,
		(void (*)(void)) wrap_pathconf,
		NULL
	},
	{ /* FILE *popen(const char *command, const char *mode); */
		"popen",
		(void (**)(void)) &real_popen,
		(void (*)(void)) wrap_popen,
		NULL
	},
	{ /* int prctl(int option, va_list ap); */
		"prctl",
		(void (**)(void)) &real_prctl,
		(void (*)(void)) wrap_prctl,
		NULL
	},
	{ /* ssize_t readlink(const char *path, char *buf, size_t bufsiz); */
		"readlink",
		(void (**)(void)) &real_readlink,
		(void (*)(void)) wrap_readlink,
		NULL
	},
	{ /* ssize_t readlinkat(int dirfd, const char *path, char *buf, size_t bufsiz); */
		"readlinkat",
		(void (**)(void)) &real_readlinkat,
		(void (*)(void)) wrap_readlinkat,
		NULL
	},
	{ /* char *realpath(const char *name, char *resolved_name); */
		"realpath",
		(void (**)(void)) &real_realpath,
		(void (*)(void)) wrap_realpath,
		"GLIBC_2.3"
	},
	{ /* int remove(const char *path); */
		"remove",
		(void (**)(void)) &real_remove,
		(void (*)(void)) wrap_remove,
		NULL
	},
	{ /* int removexattr(const char *path, const char *name); */
		"removexattr",
		(void (**)(void)) &real_removexattr,
		(void (*)(void)) wrap_removexattr,
		"GLIBC_2.3"
	},
	{ /* int rename(const char *oldpath, const char *newpath); */
		"rename",
		(void (**)(void)) &real_rename,
		(void (*)(void)) wrap_rename,
		NULL
	},
	{ /* int renameat(int olddirfd, const char *oldpath, int newdirfd, const char *newpath); */
		"renameat",
		(void (**)(void)) &real_renameat,
		(void (*)(void)) wrap_renameat,
		NULL
	},
	{ /* int renameat2(int olddirfd, const char *oldpath, int newdirfd, const char *newpath, unsigned int flags); */
		"renameat2",
		(void (**)(void)) &real_renameat2,
		(void (*)(void)) wrap_renameat2,
		NULL
	},
	{ /* int rmdir(const char *path); */
		"rmdir",
		(void (**)(void)) &real_rmdir,
		(void (*)(void)) wrap_rmdir,
		NULL
	},
	{ /* int scandir(const char *path, struct dirent ***namelist, int (*filter)(const struct dirent *), int (*compar)()); */
		"scandir",
		(void (**)(void)) &real_scandir,
		(void (*)(void)) wrap_scandir,
		NULL
	},
	{ /* int scandir64(const char *path, struct dirent64 ***namelist, int (*filter)(const struct dirent64 *), int (*compar)()); */
		"scandir64",
		(void (**)(void)) &real_scandir64,
		(void (*)(void)) wrap_scandir64,
		NULL
	},
	{ /* int setegid(gid_t egid); */
		"setegid",
		(void (**)(void)) &real_setegid,
		(void (*)(void)) wrap_setegid,
		NULL
	},
	{ /* int seteuid(uid_t euid); */
		"seteuid",
		(void (**)(void)) &real_seteuid,
		(void (*)(void)) wrap_seteuid,
		NULL
	},
	{ /* int setfsgid(gid_t fsgid); */
		"setfsgid",
		(void (**)(void)) &real_setfsgid,
		(void (*)(void)) wrap_setfsgid,
		NULL
	},
	{ /* int setfsuid(uid_t fsuid); */
		"setfsuid",
		(void (**)(void)) &real_setfsuid,
		(void (*)(void)) wrap_setfsuid,
		NULL
	},
	{ /* int setgid(gid_t gid); */
		"setgid",
		(void (**)(void)) &real_setgid,
		(void (*)(void)) wrap_setgid,
		NULL
	},
	{ /* void setgrent(void); */
		"setgrent",
		(void (**)(void)) &real_setgrent,
		(void (*)(void)) wrap_setgrent,
		NULL
	},
	{ /* int setgroups(size_t size, const gid_t *list); */
		"setgroups",
		(void (**)(void)) &real_setgroups,
		(void (*)(void)) wrap_setgroups,
		NULL
	},
	{ /* void setpwent(void); */
		"setpwent",
		(void (**)(void)) &real_setpwent,
		(void (*)(void)) wrap_setpwent,
		NULL
	},
	{ /* int setregid(gid_t rgid, gid_t egid); */
		"setregid",
		(void (**)(void)) &real_setregid,
		(void (*)(void)) wrap_setregid,
		NULL
	},
	{ /* int setresgid(gid_t rgid, gid_t egid, gid_t sgid); */
		"setresgid",
		(void (**)(void)) &real_setresgid,
		(void (*)(void)) wrap_setresgid,
		NULL
	},
	{ /* int setresuid(uid_t ruid, uid_t euid, uid_t suid); */
		"setresuid",
		(void (**)(void)) &real_setresuid,
		(void (*)(void)) wrap_setresuid,
		NULL
	},
	{ /* int setreuid(uid_t ruid, uid_t euid); */
		"setreuid",
		(void (**)(void)) &real_setreuid,
		(void (*)(void)) wrap_setreuid,
		NULL
	},
	{ /* int setuid(uid_t uid); */
		"setuid",
		(void (**)(void)) &real_setuid,
		(void (*)(void)) wrap_setuid,
		NULL
	},
	{ /* int setxattr(const char *path, const char *name, const void *value, size_t size, int xflags); */
		"setxattr",
		(void (**)(void)) &real_setxattr,
		(void (*)(void)) wrap_setxattr,
		"GLIBC_2.3"
	},
	{ /* int stat(const char *path, struct stat *buf); */
		"stat",
		(void (**)(void)) &real_stat,
		(void (*)(void)) wrap_stat,
		NULL
	},
	{ /* int stat64(const char *path, struct stat64 *buf); */
		"stat64",
		(void (**)(void)) &real_stat64,
		(void (*)(void)) wrap_stat64,
		NULL
	},
	{ /* int statvfs(const char *path, struct statvfs *buf); */
		"statvfs",
		(void (**)(void)) &real_statvfs,
		(void (*)(void)) wrap_statvfs,
		NULL
	},
	{ /* int statx(int dirfd, const char *path, int flags, unsigned int mask, struct statx *statxbuf); */
		"statx",
		(void (**)(void)) &real_statx,
		(void (*)(void)) wrap_statx,
		NULL
	},
	{ /* int symlink(const char *oldname, const char *newpath); */
		"symlink",
		(void (**)(void)) &real_symlink,
		(void (*)(void)) wrap_symlink,
		NULL
	},
	{ /* int symlinkat(const char *oldname, int dirfd, const char *newpath); */
		"symlinkat",
		(void (**)(void)) &real_symlinkat,
		(void (*)(void)) wrap_symlinkat,
		NULL
	},
	{ /* void sync(void); */
		"sync",
		(void (**)(void)) &real_sync,
		(void (*)(void)) wrap_sync,
		NULL
	},
	{ /* int sync_file_range(int fd, off64_t offset, off64_t nbytes, unsigned int flags); */
		"sync_file_range",
		(void (**)(void)) &real_sync_file_range,
		(void (*)(void)) wrap_sync_file_range,
		NULL
	},
	{ /* int syncfs(int fd); */
		"syncfs",
		(void (**)(void)) &real_syncfs,
		(void (*)(void)) wrap_syncfs,
		NULL
	},
	{ /* long syscall(long nr, va_list ap); */
		"syscall",
		(void (**)(void)) &real_syscall,
		(void (*)(void)) wrap_syscall,
		NULL
	},
	{ /* int system(const char *command); */
		"system",
		(void (**)(void)) &real_system,
		(void (*)(void)) wrap_system,
		NULL
	},
	{ /* char *tempnam(const char *template, const char *pfx); */
		"tempnam",
		(void (**)(void)) &real_tempnam,
		(void (*)(void)) wrap_tempnam,
		NULL
	},
	{ /* char *tmpnam(char *s); */
		"tmpnam",
		(void (**)(void)) &real_tmpnam,
		(void (*)(void)) wrap_tmpnam,
		NULL
	},
	{ /* int truncate(const char *path, off_t length); */
		"truncate",
		(void (**)(void)) &real_truncate,
		(void (*)(void)) wrap_truncate,
		NULL
	},
	{ /* int truncate64(const char *path, off64_t length); */
		"truncate64",
		(void (**)(void)) &real_truncate64,
		(void (*)(void)) wrap_truncate64,
		NULL
	},
	{ /* int ulckpwdf(void); */
		"ulckpwdf",
		(void (**)(void)) &real_ulckpwdf,
		(void (*)(void)) wrap_ulckpwdf,
		NULL
	},
	{ /* mode_t umask(mode_t mask); */
		"umask",
		(void (**)(void)) &real_umask,
		(void (*)(void)) wrap_umask,
		NULL
	},
	{ /* int unlink(const char *path); */
		"unlink",
		(void (**)(void)) &real_unlink,
		(void (*)(void)) wrap_unlink,
		NULL
	},
	{ /* int unlinkat(int dirfd, const char *path, int rflags); */
		"unlinkat",
		(void (**)(void)) &real_unlinkat,
		(void (*)(void)) wrap_unlinkat,
		NULL
	},
	{ /* int utime(const char *path, const struct utimbuf *buf); */
		"utime",
		(void (**)(void)) &real_utime,
		(void (*)(void)) wrap_utime,
		NULL
	},
	{ /* int utimes(const char *path, const struct timeval *times); */
		"utimes",
		(void (**)(void)) &real_utimes,
		(void (*)(void)) wrap_utimes,
		NULL
	},
	{ NULL, NULL, NULL, NULL },
};
