/* 
 * Copyright (c) 2010 Wind River Systems; see
 * guts/COPYRIGHT for information.
 *
 * SPDX-License-Identifier: LGPL-2.1-only
 *
 * static void
 * wrap_endpwent(void) {
 *	
 */

	pseudo_pwd_close();

/*	return;
 * }
 */
